@@ -107,6 +107,7 @@ GLMM_longitDA(double*       Y_c,                       /* it is in fact const, n
   const int max_LT_b  = (max_dim_b * (max_dim_b + 1)) / 2;
   const int max_Kmax_b = AK_Basic::maxArray(Kmax_b, *nClust);
 
+
  
   /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/
   /***** Data related variables                                                                             *****/
@@ -156,7 +157,7 @@ GLMM_longitDA(double*       Y_c,                       /* it is in fact const, n
 
 
   /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/
-  /***** Loop over clusters and sampled values                                                              *****/
+  /***** Declarations for looping over clusters and sampled values                                          *****/
   /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/ 
   int backs = 1;
   int iter_backs;
@@ -180,7 +181,6 @@ GLMM_longitDA(double*       Y_c,                       /* it is in fact const, n
   int nrowZiS, l_ZiS;
   const int *nP;
 
-
   double *SZitZiS_c    = NULL;
   double *SZitZiS_d    = NULL;
   int l_SZitZiS_c, l_SZitZiS_d;
@@ -212,148 +212,178 @@ GLMM_longitDA(double*       Y_c,                       /* it is in fact const, n
   int ncolX, ncolZ; 
   double *log_dets_bP;
 
-  //Rprintf((char*)("Cluster  "));
-  for (clust_lC = 0; clust_lC < *nClust; clust_lC++){          /*** loop(clust_lC) over groups to which we discriminate ***/
 
-    /***** Progress information *****/
-    Rprintf((char*)("Cluster %d\n"), clust_lC + 1);
-    //for (i = 0; i < backs; i++) Rprintf((char*)("\b"));
-    //Rprintf((char*)("%d"), clust_lC + 1);
-    //backs = int(log10(double(clust_lC + 1))) + 1;
-
-    /***** Fill log 2pi part of log_dets_b *****/
-    log_dets_bP = log_dets_b + 1;
-    for (k = 0; k < *Kmax_b_cl; k++){
-      *log_dets_bP = -(*dim_b_cl) * M_LN_SQRT_2PI;
-      log_dets_bP += 2;
-    }
-
-    /***** Set-up some pointers *****/    
-    scale_b_cl = shift_b_cl + *dim_b_cl;
-
-    /***** Set-up data related variables *****/    
-    AK_Basic::cumsum(cumq_ri_cl, q_ri_cl, R);
-    ncolX = AK_Basic::sum(p_cl, R);
-    ncolZ = AK_Basic::sum(q_cl, R);
-
-    /***** Compute eta_zs *****/
-    GLMM::linear_predictor_zs(eta_zs, Z_cl, shift_b_cl, q_cl, randIntcpt_cl, n, &R, I, dim_b_cl, cumq_ri_cl);
-
-    /***** Set-up Zresp *****/
-    *Zresp = Z_cl;
-    for (s = 1; s < R; s++) Zresp[s] = Zresp[s-1] + q_cl[s-1] * N_s; 
-
-    /***** Total space needed for SZitZiS_c and SZitZiS_d *****/
-    /***** Allocate this space                            *****/
-    l_SZitZiS_c = 0;
-    for (s = 0; s < *R_c; s++) l_SZitZiS_c += N_s * ((q_ri_cl[s] * (q_ri_cl[s] + 1)) / 2);       
-    SZitZiS_c = Calloc(l_SZitZiS_c > 0 ? l_SZitZiS_c : 1, double);
-
-    l_SZitZiS_d = 0;
-    for (s = *R_c; s < *R_c + *R_d; s++) l_SZitZiS_d += N_s * ((q_ri_cl[s] * (q_ri_cl[s] + 1)) / 2);       
-    SZitZiS_d = Calloc(l_SZitZiS_d > 0 ? l_SZitZiS_d : 1, double);   
-
-    /***** Calculate matrices SZitZiS_c and SZitZiS_d *****/
-    GLMM::create_SZitZiS_4longitDA(SZitZiS_c, SZitZiS_d, ZrespP, Zresp, scale_b_cl, q_cl, randIntcpt_cl, R_c, R_d, I, n);
-    
-    /***** Compute Zi*S matrices for each observation we will predict       *****/
-         /** First, calculate number of columns in one block for one response type  **/
-         /** = (1+2+...+n[0] + ... + (1+2+...+n[I-1])) **/  
-    nrowZiS = 0;
-    nP = n;
-    for (i = 0; i < *I; i++){
-      nrowZiS += (*nP * (1 + *nP)) / 2;
-      nP++;
-    }
-         /** Second, calculate total length of the space to store S*t(Zi) and allocate needed space **/
-    l_ZiS = nrowZiS * *dim_b_cl;
-    ZiS = Calloc(l_ZiS, double);
-         /** Third, calculate Zi*S matrices (order of storage will be the same as for SZitZiS)   **/
-         /** REMARK:  Like Z and X matrices, matrices ZiS are stored in ROW major order          **/
-    GLMM::create_ZiS(ZiS, ZrespP, Zresp, scale_b_cl, q_cl, randIntcpt_cl, &R, I, n);
+    /***** Distributions *****/
+  int allGaussIdent = 1;
+  for (s = 0; s < R; s++){
+    if (dist[s] != GLMM::GAUSS_IDENTITY) allGaussIdent = 0;
+  }
 
 
-    /***** Loop over sampled values *****/
-    iter_backs = 1;
-    Rprintf((char*)("Iteration  "));
-    for (iter_lC = 1; iter_lC <= *keepMCMC_cl; iter_lC++){
+  /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/
+  /***** Loop over clusters and sampled values in case all variables are CONTINUOUS                         *****/
+  /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/ 
 
-      /*** Progress information ***/
-      if (!(iter_lC % *info) || iter_lC == *keepMCMC_cl){
-        for (i = 0; i < iter_backs; i++) Rprintf((char*)("\b"));
-        Rprintf((char*)("%d"), iter_lC);
-        iter_backs = int(log10(double(iter_lC))) + 1;
+  if (allGaussIdent){
+
+    //Rprintf((char*)("Cluster  "));
+    for (clust_lC = 0; clust_lC < *nClust; clust_lC++){          /*** loop(clust_lC) over groups to which we discriminate ***/
+
+      /***** Progress information *****/
+      Rprintf((char*)("Cluster %d\n"), clust_lC + 1);
+      //for (i = 0; i < backs; i++) Rprintf((char*)("\b"));
+      //Rprintf((char*)("%d"), clust_lC + 1);
+      //backs = int(log10(double(clust_lC + 1))) + 1;
+
+      /***** Fill log 2pi part of log_dets_b *****/
+      log_dets_bP = log_dets_b + 1;
+      for (k = 0; k < *Kmax_b_cl; k++){
+        *log_dets_bP = -(*dim_b_cl) * M_LN_SQRT_2PI;
+        log_dets_bP += 2;
       }
 
-      /*** Main computation ***/
-      GLMM::longitPred_nmix_gauss(pi_marg_cl, pi_cond_cl, pi_ranef_cl,
-                                  eta_fixedresp, eta_random, 
-                                  log_dets_b, dworkPred, iworkPred,
-                                  Y_crespP, Y_drespP, eta_fixedrespP, eta_zsrespP, ZrespP, err,
-                                  Y_cresp, Y_dresp, eta_zsresp, X_cl, Zresp, SZitZiS_c, ZiS, shift_b_cl, scale_b_cl, 
-                                  p_cl, fixedIntcpt_cl, 
-                                  q_cl, randIntcpt_cl, q_ri_cl, cumq_ri_cl, dim_b_cl, LT_b_cl,
-                                  R_c, R_d, I, n, &max_n, beta, sigma_eps, K_b, w_b, mu_b, Li_b);
+      /***** Set-up some pointers *****/    
+      scale_b_cl = shift_b_cl + *dim_b_cl;
+
+      /***** Set-up data related variables *****/    
+      AK_Basic::cumsum(cumq_ri_cl, q_ri_cl, R);
+      ncolX = AK_Basic::sum(p_cl, R);
+      ncolZ = AK_Basic::sum(q_cl, R);
+
+      /***** Compute eta_zs *****/
+      GLMM::linear_predictor_zs(eta_zs, Z_cl, shift_b_cl, q_cl, randIntcpt_cl, n, &R, I, dim_b_cl, cumq_ri_cl);
+
+      /***** Set-up Zresp *****/
+      *Zresp = Z_cl;
+      for (s = 1; s < R; s++) Zresp[s] = Zresp[s-1] + q_cl[s-1] * N_s; 
+
+      /***** Total space needed for SZitZiS_c and SZitZiS_d *****/
+      /***** Allocate this space                            *****/
+      l_SZitZiS_c = 0;
+      for (s = 0; s < *R_c; s++) l_SZitZiS_c += N_s * ((q_ri_cl[s] * (q_ri_cl[s] + 1)) / 2);       
+      SZitZiS_c = Calloc(l_SZitZiS_c > 0 ? l_SZitZiS_c : 1, double);
+
+      l_SZitZiS_d = 0;
+      for (s = *R_c; s < *R_c + *R_d; s++) l_SZitZiS_d += N_s * ((q_ri_cl[s] * (q_ri_cl[s] + 1)) / 2);       
+      SZitZiS_d = Calloc(l_SZitZiS_d > 0 ? l_SZitZiS_d : 1, double);   
+
+      /***** Calculate matrices SZitZiS_c and SZitZiS_d *****/
+      GLMM::create_SZitZiS_4longitDA(SZitZiS_c, SZitZiS_d, ZrespP, Zresp, scale_b_cl, q_cl, randIntcpt_cl, R_c, R_d, I, n);
+    
+      /***** Compute Zi*S matrices for each observation we will predict       *****/
+           /** First, calculate number of columns in one block for one response type  **/
+           /** = (1+2+...+n[0] + ... + (1+2+...+n[I-1])) **/  
+      nrowZiS = 0;
+      nP = n;
+      for (i = 0; i < *I; i++){
+        nrowZiS += (*nP * (1 + *nP)) / 2;
+        nP++;
+      }
+           /** Second, calculate total length of the space to store S*t(Zi) and allocate needed space **/
+      l_ZiS = nrowZiS * *dim_b_cl;
+      ZiS = Calloc(l_ZiS, double);
+           /** Third, calculate Zi*S matrices (order of storage will be the same as for SZitZiS)   **/
+           /** REMARK:  Like Z and X matrices, matrices ZiS are stored in ROW major order          **/
+      GLMM::create_ZiS(ZiS, ZrespP, Zresp, scale_b_cl, q_cl, randIntcpt_cl, &R, I, n);
+
+
+      /***** Loop over sampled values *****/
+      iter_backs = 1;
+      Rprintf((char*)("Iteration  "));
+      for (iter_lC = 1; iter_lC <= *keepMCMC_cl; iter_lC++){
+
+        /*** Progress information ***/
+        if (!(iter_lC % *info) || iter_lC == *keepMCMC_cl){
+          for (i = 0; i < iter_backs; i++) Rprintf((char*)("\b"));
+          Rprintf((char*)("%d"), iter_lC);
+          iter_backs = int(log10(double(iter_lC))) + 1;
+        }
+
+        /*** Main computation ***/
+        GLMM::longitPred_nmix_gauss(pi_marg_cl, pi_cond_cl, pi_ranef_cl,
+                                    eta_fixedresp, eta_random, 
+                                    log_dets_b, dworkPred, iworkPred,
+                                    Y_crespP, Y_drespP, eta_fixedrespP, eta_zsrespP, ZrespP, err,
+                                    Y_cresp, Y_dresp, eta_zsresp, X_cl, Zresp, SZitZiS_c, ZiS, shift_b_cl, scale_b_cl, 
+                                    p_cl, fixedIntcpt_cl, 
+                                    q_cl, randIntcpt_cl, q_ri_cl, cumq_ri_cl, dim_b_cl, LT_b_cl,
+                                    R_c, R_d, I, n, &max_n, beta, sigma_eps, K_b, w_b, mu_b, Li_b);
                                 
-      /*** Shift pointers ***/  
-      beta      += *l_beta_cl;
-      sigma_eps += *R_c;
-      w_b       += *K_b;
-      mu_b      += *K_b * *dim_b_cl;
-      Li_b      += *K_b * *LT_b_cl;
-      K_b++;
+        /*** Shift pointers ***/  
+        beta      += *l_beta_cl;
+        sigma_eps += *R_c;
+        w_b       += *K_b;
+        mu_b      += *K_b * *dim_b_cl;
+        Li_b      += *K_b * *LT_b_cl;
+        K_b++;
+      }
+      Rprintf((char*)("\n"));
+
+      /***** Compute posterior predictive means of f(..|..)             *****/
+      /***** Correct pi_ranef for scaling (divide by prod(scale_b^2))   *****/
+      det_S = AK_Basic::prod(scale_b_cl, *dim_b_cl);
+
+      pi_margP  = pi_marg_cl;
+      pi_condP  = pi_cond_cl;
+      pi_ranefP = pi_ranef_cl;
+      for (i = 0; i < N_s; i++){
+        *pi_margP /= *keepMCMC_cl;
+        *pi_condP /= *keepMCMC_cl;
+        *pi_ranefP /= *keepMCMC_cl;
+        *pi_ranefP /= det_S;
+
+        pi_margP++;
+        pi_condP++;
+        pi_ranefP++;
+      }
+
+      /***** Shift pointers *****/
+      pi_marg_cl  = pi_margP;
+      pi_cond_cl  = pi_condP;
+      pi_ranef_cl = pi_ranefP;
+
+      if (ncolX) X_cl += N_s * ncolX;
+      else       X_cl++;
+      if (ncolZ) Z_cl += N_s * ncolZ;
+      else       Z_cl++;
+
+      shift_b_cl = scale_b_cl + *dim_b_cl;
+
+      keepMCMC_cl++;
+      dim_b_cl++;
+      LT_b_cl++;
+      l_beta_cl++;
+      Kmax_b_cl++;
+
+      p_cl           += R;
+      p_fi_cl        += R;
+      fixedIntcpt_cl += R;
+      q_cl           += R;
+      q_ri_cl        += R;
+      randIntcpt_cl  += R;
+
+      /***** Cleaning *****/
+      Free(ZiS);
+      Free(SZitZiS_c);
+      Free(SZitZiS_d);
     }
-    Rprintf((char*)("\n"));
+    Rprintf((char*)("\n"));  
 
-    /***** Compute posterior predictive means of f(..|..)             *****/
-    /***** Correct pi_ranef for scaling (divide by prod(scale_b^2))   *****/
-    det_S = AK_Basic::prod(scale_b_cl, *dim_b_cl);
 
-    pi_margP  = pi_marg_cl;
-    pi_condP  = pi_cond_cl;
-    pi_ranefP = pi_ranef_cl;
-    for (i = 0; i < N_s; i++){
-      *pi_margP /= *keepMCMC_cl;
-      *pi_condP /= *keepMCMC_cl;
-      *pi_ranefP /= *keepMCMC_cl;
-      *pi_ranefP /= det_S;
+  }       // end of if (allGaussIdent)
 
-      pi_margP++;
-      pi_condP++;
-      pi_ranefP++;
-    }
 
-    /***** Shift pointers *****/
-    pi_marg_cl  = pi_margP;
-    pi_cond_cl  = pi_condP;
-    pi_ranef_cl = pi_ranefP;
+  /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/
+  /***** Loop over clusters and sampled values in case there are some DISCRETE variables                    *****/
+  /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/ 
 
-    if (ncolX) X_cl += N_s * ncolX;
-    else       X_cl++;
-    if (ncolZ) Z_cl += N_s * ncolZ;
-    else       Z_cl++;
+  else{
 
-    shift_b_cl = scale_b_cl + *dim_b_cl;
+    *err = 1;
+    error("%s: You should never get here... C++ function GLMM_longitDA2 was to be called... Only AK can solve this ;-).\n", fname);
 
-    keepMCMC_cl++;
-    dim_b_cl++;
-    LT_b_cl++;
-    l_beta_cl++;
-    Kmax_b_cl++;
+  }       // end of else from if (allGaussIdent)
 
-    p_cl           += R;
-    p_fi_cl        += R;
-    fixedIntcpt_cl += R;
-    q_cl           += R;
-    q_ri_cl        += R;
-    randIntcpt_cl  += R;
-
-    /***** Cleaning *****/
-    Free(ZiS);
-    Free(SZitZiS_c);
-    Free(SZitZiS_d);
-  }
-  Rprintf((char*)("\n"));  
 
 
   /***** %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *****/
@@ -362,57 +392,35 @@ GLMM_longitDA(double*       Y_c,                       /* it is in fact const, n
   //Rprintf((char*)("Start cleaning.\n"));  
   Free(iworkPred);
   Free(dworkPred);
-  //Rprintf((char*)("iworkPred, dworkPred cleaned.\n"));  
 
   Free(log_dets_b);
-  //Rprintf((char*)("log_dets_b cleaned.\n"));  
 
   Free(q_ri);
-  //Rprintf((char*)("q_ri cleaned.\n"));  
   Free(p_fi);
-  //Rprintf((char*)("p_fi cleaned.\n"));  
   Free(LT_b);
-  //Rprintf((char*)("LT_b cleaned.\n"));  
   Free(dim_b);
-  //Rprintf((char*)("dim_b cleaned.\n"));  
   Free(l_beta);
-  //Rprintf((char*)("l_beta cleaned.\n"));  
   Free(cumq_ri_cl);
-  //Rprintf((char*)("cumq_ri_cl cleaned.\n"));  
 
   Free(Zresp);
-  //Rprintf((char*)("Zresp cleaned.\n"));  
   Free(ZrespP);
-  //Rprintf((char*)("ZrespP cleaned.\n"));  
   Free(eta_fixedresp);
-  //Rprintf((char*)("eta_fixedresp cleaned.\n"));  
   Free(eta_fixedrespP);
-  //Rprintf((char*)("eta_fixedrespP cleaned.\n"));  
   Free(eta_zsresp);
-  //Rprintf((char*)("eta_zsresp cleaned.\n"));  
   Free(eta_zsrespP);
-  //Rprintf((char*)("eta_zsrespP cleaned.\n"));  
   if (*R_c){
     Free(Y_cresp);
-    //Rprintf((char*)("Y_cresp cleaned.\n"));   
     Free(Y_crespP);
-    //Rprintf((char*)("Y_crespP cleaned.\n"));  
   }
   if (*R_d){
     Free(Y_dresp);
-    //Rprintf((char*)("Y_dresp cleaned.\n"));  
     Free(Y_drespP);
-    //Rprintf((char*)("Y_drespP cleaned.\n"));  
   }
 
   Free(eta_zs);
-  //Rprintf((char*)("eta_zs cleaned.\n"));  
   Free(eta_random);
-  //Rprintf((char*)("eta_random cleaned.\n"));  
   Free(eta_fixed);  
-  //Rprintf((char*)("eta_fixed cleaned.\n"));  
 
-  //Rprintf((char*)("Leaving C++.\n"));  
   return;
 }
 
